package App::optex;
use strict;
use warnings;

our $VERSION = "v0.0.13";

1;

__END__

=encoding utf8

=head1 NAME

optex - optex command module direcrtory

=head1 SYNOPSIS

    B<optex> command [B<-M>I<module>] ...

    or make symbolic link to B<optex> in the name of target command.

=head1 DESCRIPTION

This directory contains modules for B<optex> command.  Unlike others,
these modules are loaded from top of package name space.

=head1 LICENSE

Copyright (C) Kaz Utashiro.

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself.

=head1 AUTHOR

Kaz Utashiro E<lt>kaz@utashiro.comE<gt>

=cut
